/** \file EndpointWrap.h
	\brief Declarations for the wrappers of the native structures
	to do with Endpoints
	\cond DEV
*/

#ifndef AUDRECORDCLI_ENDPOINTWRAP_H
#define AUDRECORDCLI_ENDPOINTWRAP_H

#pragma once

namespace AudRecordCLI
{
	/** Wraps an Endpoint structure */
	public ref struct Endpoint
	{
		/** The description of this endpoint. Get is public, set is internal */
		property String^ Name
		{
			String^ get()
			{
				return name;
			}
		internal:
			void set(String^ newName)
			{
				name = newName;
			}
		}
	internal:
		/** An object returned from Marshal::GetObjectFromIUnknown on the native IMMDevice pointer */
		property Object^ Interface;
	private:
		/** The endpoint name, the backing store of the property */
		String^ name;
	};

	/** Typedef of a collection of endpoints for simplification purposes */
	typedef List<Endpoint^> EndpointList;

	/** Static class to wrap endpoint related functions */
	public value struct Endpoints abstract sealed
	{
		/** Calls the native GetAudioEndpoints() and returns a list of managed wrappers */
		static EndpointList^ Get();
	};
}

/** \endcond */

#endif
