#define _WIN32_WINNT 0x0600
// for the c friendly com method macros
#define COBJMACROS
#include <windows.h>
#include <objbase.h>
#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include "../exports.h"
#ifdef _WIN64
#pragma comment(lib, "../binaries/x64/audrecordlib.lib")
#else
#pragma comment(lib, "../binaries/x86/audrecordlib.lib")
#endif

void ErrorExit(HRESULT hr, const char* fn)
{
	printf("%s failed with error %#x\n", hr);
	exit(0);
}

int main()
{
	CodecConfigs* pConfigs = NULL;
	HRESULT hr = S_OK;
	Codec* pCodec = NULL;
	CodecConfiguration* pConfig = NULL;
	IAudRecord* pRecorder = NULL;
	EndpointArray* pEndpoints = NULL;
	Endpoint* pEndpointToUse = NULL;
	PVOID pCookie = NULL;
	wchar_t* pTempFile = NULL;

	// initialize COM, important
	CoInitializeEx(NULL, COINIT_MULTITHREADED | COINIT_DISABLE_OLE1DDE);
	// get a list of codecs
	if(FAILED(hr = GetWMACodecConfigs(&pConfigs)))
	{
		ErrorExit(hr, "GetWMACodecConfigs");
	}
	// pick one, here it doesn't matter
	pCodec = pConfigs->pCodecs[0];
	pConfig = &pCodec->configs[0];
	printf("Using codec %S and config %S\n", pCodec->name, pConfig->pName);
	
	// Create the audrecord interface using the configuration
	if(FAILED(hr = CreateAudRecordInterface(pConfig, &pRecorder)))
	{
		FreeWMACodecConfigs(pConfigs);
		ErrorExit(hr, "CreateAudRecordInterface");
	}
	// finished with these now
	FreeWMACodecConfigs(pConfigs);

	// get a list of audio devices
	if(FAILED(hr = GetAudioEndpoints(&pEndpoints)))
	{
		IAudRecord_Release(pRecorder);
		ErrorExit(hr, "GetAudioEndpoints");
	}
	// pick one
	pEndpointToUse = &pEndpoints->outputs[0];
	printf("For output device %S\n", pEndpointToUse->pName);

	// add a file as a recording output
	pTempFile = _wtempnam(NULL, L"ars");
	printf("Recording to file %S\n", pTempFile);
	hr = IAudRecord_AddFileTarget(pRecorder, pTempFile, &pCookie);
	if(FAILED(hr))
	{
		_wunlink(pTempFile);
		free(pTempFile);
		FreeAudioEndpoints(pEndpoints);
		IAudRecord_Release(pRecorder);
		ErrorExit(hr, "IAudRecord_AddFileTarget");
	}

	puts("Beginning recording...");
	// go record to the file, using the device selected
	if(FAILED(hr = IAudRecord_Record(pRecorder, pEndpointToUse)))
	{
		FreeAudioEndpoints(pEndpoints);
		IAudRecord_Release(pRecorder);
		_wunlink(pTempFile);
		free(pTempFile);
		ErrorExit(hr, "IAudRecord_Record");
	}
	puts("Recording started successfully, press enter to stop and exit");
	getchar();
	// user wanted to stop, so stop and do all the cleanup
	IAudRecord_Stop(pRecorder);
	IAudRecord_Release(pRecorder);
	FreeAudioEndpoints(pEndpoints);
	printf("Enter \'d\' to delete the recorded file, any other to keep it: ");
	if(tolower(getchar()) == 'd')
	{
		_wunlink(pTempFile);
	}
	free(pTempFile);
	CoUninitialize();
}
