#include "stdafx.h"
#include "AudRecordTest.h"
#include "resource.h"
#include "../exports.h"
#include <cstdlib>
#include <cassert>
#include "CreateSinkDlg.h"

struct AudRecordData
{
	WinType<EndpointArray*, void> pEndpoints;
	WinType<CodecConfigs*, void> pCodecs;
	ComType<IAudRecord> pRecord;

	AudRecordData(WinType<EndpointArray*, void>& pInEndpoints, WinType<CodecConfigs*, void>& pInCodecs)
		: pEndpoints(pInEndpoints.Release(), &FreeAudioEndpoints),
		pCodecs(pInCodecs.Release(), &FreeWMACodecConfigs),
		pRecord(NULL)
	{}

	~AudRecordData()
	{
		if(pRecord)
		{
			pRecord->Stop();
		}
	}
};

inline AudRecordData* GetDialogData(HWND hwnd)
{
	return reinterpret_cast<AudRecordData*>(GetWindowLongPtr(hwnd, DWLP_USER));
}

INT_PTR CALLBACK DlgProc(HWND, UINT, WPARAM, LPARAM);

int APIENTRY wWinMain(HINSTANCE hInstance, HINSTANCE, LPWSTR, int)
{
	srand(GetTickCount());
	CoInitializeEx(NULL, COINIT_APARTMENTTHREADED | COINIT_DISABLE_OLE1DDE);
	InitCommonControls();

	INT_PTR ret = DialogBox(hInstance, MAKEINTRESOURCE(DLG_MAIN), NULL, &DlgProc);
	if(ret == -1)
	{
		return 1;
	}
	CoUninitialize();
	DumpUnfreedAllocations();
	return ret;
}

BOOL OnInitDialog(HWND hDlg, HWND hwndFocus, LPARAM)
{
	HWND hMixerList = WINDOW(COMBO_MIXER);
	HWND hCodecList = WINDOW(COMBO_CODEC);
	HWND hListTargets = WINDOW(LISTVIEW_TARGETS);
	WinType<EndpointArray*, void> pEndpointArray(NULL, &FreeAudioEndpoints);
	WinType<CodecConfigs*, void> pCodecs(NULL, &FreeWMACodecConfigs);
	AudRecordData* pAudData = NULL;

	LVCOLUMN col = {0};
	RECT listClient;
	GetClientRect(hListTargets, &listClient);
	col.mask = LVCF_WIDTH | LVCF_TEXT;
	col.cx = listClient.right;
	// we're not displaying column headers, so no need to set this
	col.pszText = L"";
	ListView_InsertColumn(hListTargets, 0, &col);

	HRESULT hr = GetAudioEndpoints(&pEndpointArray);
	if(FAILED(hr))
	{
		goto failedHR;
	}
	if(FAILED(hr = GetWMACodecConfigs(&pCodecs)))
	{
		goto failedHR;
	}
	pAudData = new(std::nothrow) AudRecordData(pEndpointArray, pCodecs);
	if(!pAudData)
	{
		hr = E_OUTOFMEMORY;
		goto failedHR;
	}
	// add the endpoints to their list
	EndpointArray* pArrayEnd = *(pAudData->pEndpoints);
	Endpoint* pIterEndpoint = pArrayEnd->outputs, *pEndEndpoint = pIterEndpoint + pArrayEnd->num;
	WPARAM index = 0;
	while(pIterEndpoint != pEndEndpoint)
	{
		ComboBox_InsertString(hMixerList, index++, pIterEndpoint->pName);
		++pIterEndpoint;
	}
	// and the codec names to theirs
	CodecConfigs* pConfigs = *(pAudData->pCodecs);
	Codec** pIterCodec = pConfigs->pCodecs, **pEndCodec = pIterCodec + pConfigs->num;
	index = 0;
	while(pIterCodec != pEndCodec)
	{
		ComboBox_InsertString(hCodecList, index++, (*pIterCodec)->name);
		++pIterCodec;
	}
	ListView_SetExtendedListViewStyle(hListTargets, LVS_EX_FULLROWSELECT);
	SetFocus(hMixerList);
	SetWindowLongPtr(hDlg, DWLP_USER, reinterpret_cast<LPARAM>(pAudData));
	// we've set the focus, FALSE prevents Windows from settings its own
	return FALSE;

failedHR:
	delete pAudData;
	EndDialog(hDlg, hr);
	return FALSE;
}

void PopulateConfigList(HWND hDlg, int index)
{
	AudRecordData* pAudData = GetDialogData(hDlg);
	Codec* pCodecSelected = (*pAudData->pCodecs)->pCodecs[index];
	CodecConfiguration* pConfigIter = pCodecSelected->configs, *pConfigEnd = pConfigIter + pCodecSelected->numConfigs;
	HWND hConfigList = WINDOW(COMBO_CONFIG);
	ComboBox_ResetContent(hConfigList);
	WPARAM configIndex = 0;
	while(pConfigIter != pConfigEnd)
	{
		ComboBox_InsertString(hConfigList, configIndex++, pConfigIter->pName);
		++pConfigIter;
	}
}

HRESULT CreateOrChangeAudRecordConfig(HWND hDlg)
{
	AudRecordData* pData = GetDialogData(hDlg);
	int codecIndex = SendDlgItemMessage(hDlg, COMBO_CODEC, CB_GETCURSEL, 0, 0);
	int configIndex = SendDlgItemMessage(hDlg, COMBO_CONFIG, CB_GETCURSEL, 0, 0);
	Codec* pCodecTemp = (*pData->pCodecs)->pCodecs[codecIndex];
	CodecConfiguration* pConfig = pCodecTemp->configs + configIndex;
	HRESULT hr = S_OK;
	if(!pData->pRecord)
	{
		ComType<IAudRecord> pRecord;
		hr = CreateAudRecordInterface(pConfig, &pRecord);
		if(SUCCEEDED(hr))
		{
			pData->pRecord = pRecord;
		}
	}
	else
	{
		hr = pData->pRecord->ChangeCodec(pConfig);
	}
	return hr;
}

HRESULT StartRecording(HWND hDlg)
{
	AudRecordData* pData = GetDialogData(hDlg);
	ASSERT(pData->pRecord);
	int mixerIndex = SendDlgItemMessage(hDlg, COMBO_MIXER, CB_GETCURSEL, 0, 0);
	Endpoint* pMixer = (*pData->pEndpoints)->outputs + mixerIndex;
	return pData->pRecord->Record(pMixer);
}

HRESULT PauseRecording(HWND hDlg, AudPausedState& paused)
{
	AudRecordData* pData = GetDialogData(hDlg);
	ASSERT(pData->pRecord);
	return pData->pRecord->TogglePause(&paused);
}

HRESULT StopRecording(HWND hDlg)
{
	AudRecordData* pData = GetDialogData(hDlg);
	ASSERT(pData->pRecord);
	return pData->pRecord->Stop();
}

void ChangePauseButtonText(HWND hDlg, AudPausedState recording)
{
	static const DWORD ids[] = {STR_PAUSE, STR_RESUME};
	std::wstring newText = GetModuleString(GetWindowInstance(hDlg), ids[recording == AudRecording]);
	SetWindowText(WINDOW(BTN_PAUSE), newText.c_str());
}

void AddSink(HWND hDlg)
{
	AudRecordData* pAudData = GetDialogData(hDlg);
	CreateSinkDetails newSink = {};
	HMODULE hThis = GetWindowInstance(hDlg);
	INT_PTR value = DialogBoxParam(
		hThis,
		MAKEINTRESOURCE(DLG_SINK_CREATE),
		hDlg,
		&SinkCreateDlgProc,
		reinterpret_cast<LPARAM>(&newSink)
	);
	if(value)
	{
		HRESULT hr = S_OK;
		PVOID pCookie = NULL;
		std::wstring description;
		if(newSink.dwPort)
		{
			description = GetModuleString(hThis, STR_NETWORKTARGET);
			LPWSTR pUrl = NULL;
			if(SUCCEEDED(hr = pAudData->pRecord->AddNetworkBroadcastTarget(&newSink.dwPort, 0, &pUrl, &pCookie)))
			{
				description += pUrl;
				CoTaskMemFree(pUrl);
			}
		}
		else
		{
			description = GetModuleString(hThis, STR_FILETARGET);
			description += newSink.file;
			hr = pAudData->pRecord->AddFileTarget(newSink.file.c_str(), &pCookie);
		}
		if(SUCCEEDED(hr))
		{
			LVITEM lv = {0};
			lv.mask = LVIF_PARAM | LVIF_TEXT;
			lv.lParam = reinterpret_cast<LPARAM>(pCookie);
			// this won't modify the string
			lv.pszText = const_cast<LPWSTR>(description.c_str());
			ListView_InsertItem(WINDOW(LISTVIEW_TARGETS), &lv);

			if(ComboBox_GetCurSel(WINDOW(COMBO_CONFIG)) != CB_ERR)
			{
				EnableWindow(WINDOW(BTN_RECORD), TRUE);
			}
		}
	}
}

void RemoveSink(HWND hDlg)
{
	AudRecordData* pAudData = GetDialogData(hDlg);
	HWND hListView = WINDOW(LISTVIEW_TARGETS);
	int index;
	while((index = ListView_GetNextItem(hListView, -1, LVNI_SELECTED)) != -1)
	{
		LVITEM item = {0};
		item.iItem = index;
		item.mask = LVIF_PARAM;
		ListView_GetItem(hListView, &item);
		PVOID pCookie = reinterpret_cast<PVOID>(item.lParam);
		if(SUCCEEDED(pAudData->pRecord->RemoveTarget(pCookie)))
		{
			ListView_DeleteItem(hListView, index);
		}
	}
	if(ListView_GetItemCount(hListView) <= 0)
	{
		EnableWindow(WINDOW(BTN_RECORD), FALSE);
		SetFocus(WINDOW(BTN_ADDSINK));
	}
	EnableWindow(WINDOW(BTN_DELSINK), FALSE);
}

void OnCommand(HWND hDlg, int id, HWND hwndCtl, UINT codeNotify)
{
	bool handled = false;
	switch(id)
	{
		case BTN_RECORD:
		{
			HRESULT hr = StartRecording(hDlg);
			if(SUCCEEDED(hr))
			{
				EnableWindow(WINDOW(BTN_PAUSE), TRUE);
				EnableWindow(WINDOW(BTN_STOP), TRUE);
				EnableWindow(hwndCtl, FALSE);
			}
			else
			{
				LOG_TEXT("Failed to start the recording %#x", hr);
			}
			++handled;
		}
		break;
		case BTN_PAUSE:
		{
			AudPausedState running;
			HRESULT hr = PauseRecording(hDlg, running);
			if(SUCCEEDED(hr))
			{
				ChangePauseButtonText(hDlg, running);
			}
			else
			{
				LOG_TEXT("Failed to pause/resume the recording %#x", hr);
			}
			++handled;
		}
		break;
		case BTN_STOP:
		{
			HRESULT hr = StopRecording(hDlg);
			if(SUCCEEDED(hr))
			{
				HWND hBtnPause = WINDOW(BTN_PAUSE);
				ChangePauseButtonText(hDlg, AudPaused);
				EnableWindow(WINDOW(BTN_RECORD), TRUE);
				EnableWindow(hBtnPause, FALSE);
				EnableWindow(hwndCtl, FALSE);
			}
			else
			{
				LOG_TEXT("Failed to stop the recording %#x", hr);
			}
			++handled;
		}
		break;
		case BTN_ADDSINK:
		{
			AddSink(hDlg);
			++handled;
		}
		break;
		case BTN_DELSINK:
		{
			RemoveSink(hDlg);
			++handled;
		}
		break;
	}
	if(!handled)
	{
		switch(codeNotify)
		{
			case CBN_SELCHANGE:
			{
				if(id == COMBO_CODEC)
				{
					PopulateConfigList(hDlg, ComboBox_GetCurSel(hwndCtl));
				}
				else if(id == COMBO_CONFIG)
				{
					BOOL hasSel = (ComboBox_GetCurSel(hwndCtl) != CB_ERR);
					BOOL hasMixer = (ComboBox_GetCurSel(WINDOW(COMBO_MIXER)) != CB_ERR);
					if(hasSel)
					{
						HRESULT hr = CreateOrChangeAudRecordConfig(hDlg);
						if(FAILED(hr))
						{
							LOG_TEXT("Failed to create or change the codec configuration %#x", hr);
						}
					}
					BOOL hasTargets = (ListView_GetItemCount(WINDOW(LISTVIEW_TARGETS)) > 0);
					// allow recording if there's a codec config selected
					// and some targets defined
					EnableWindow(WINDOW(BTN_RECORD), hasSel & hasTargets & hasMixer);
					// allow sinks to be added if a config is set
					EnableWindow(WINDOW(BTN_ADDSINK), hasSel);
				}
			}
			break;
		}
	}
}

void OnClose(HWND hDlg)
{
	AudRecordData* pData = GetDialogData(hDlg);
	if(pData->pRecord)
	{
		StopRecording(hDlg);
	}
	delete pData;
	EndDialog(hDlg, 0);
}

BOOL OnNotify(HWND hDlg, int /*code*/, NMHDR* pHdr)
{
	switch(pHdr->code)
	{
		case LVN_ITEMCHANGED:
		{
			if(pHdr->idFrom == LISTVIEW_TARGETS)
			{
				int count = ListView_GetSelectedCount(pHdr->hwndFrom);
				EnableWindow(WINDOW(BTN_DELSINK), count > 0);
			}
		}
		break;
		default:
		{
			LOG_TEXT("Received message %li (%#x)", pHdr->code, pHdr->code);
		}
	}
	return FALSE;
}

INT_PTR CALLBACK DlgProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		HANDLE_MSG(hwnd, WM_COMMAND, &OnCommand);
		HANDLE_MSG(hwnd, WM_NOTIFY, &OnNotify);
		HANDLE_MSG(hwnd, WM_INITDIALOG, &OnInitDialog);
		HANDLE_MSG(hwnd, WM_CLOSE, &OnClose);
	}
	return FALSE;
}
