#include "stdafx.h"
#include "CreateSinkDlg.h"
#include "resource.h"
#include <shlobj.h>
#include <cstdlib>

static BOOL OnInitDlg(HWND hDlg, HWND hwndFocus, LPARAM lParam)
{
	SetWindowLongPtr(hDlg, DWLP_USER, lParam);
	SetFocus(WINDOW(EDIT_FILE_PATH));
	return FALSE; // to stop Windows resetting the focus
}

static void GetSaveFileName(HWND hDlg)
{
	ComType<IFileSaveDialog> pSaveDlg;
	HRESULT hr = CoCreateInstance(CLSID_FileSaveDialog, NULL, CLSCTX_INPROC_SERVER, IID_PPV_ARGS(&pSaveDlg));
	if(FAILED(hr))
	{
		return;
	}
	COMDLG_FILTERSPEC fSpecs[2];
	HMODULE hMod = GetWindowInstance(hDlg);
	std::wstring wmaFileStr = GetModuleString(hMod, STR_WMAFILES);
	std::wstring allFileStr = GetModuleString(hMod, STR_ALLFILES);
	fSpecs[0].pszName = wmaFileStr.c_str();
	fSpecs[0].pszSpec = L"*.wma";
	fSpecs[1].pszName = allFileStr.c_str();
	fSpecs[1].pszSpec = L"*.*";
	pSaveDlg->SetFileTypes(2, fSpecs);
	pSaveDlg->SetDefaultExtension(L"wma");
	pSaveDlg->SetOptions(FOS_OVERWRITEPROMPT | FOS_PATHMUSTEXIST | FOS_NOREADONLYRETURN | FOS_FORCEFILESYSTEM);
	hr = pSaveDlg->Show(hDlg);
	if(FAILED(hr))
	{
		return;
	}
	ComType<IShellItem> pItem;
	if(FAILED(hr = pSaveDlg->GetResult(&pItem)))
	{
		return;
	}
	LPWSTR pName = NULL;
	if(FAILED(hr = pItem->GetDisplayName(SIGDN_FILESYSPATH, &pName)))
	{
		return;
	}
	SetWindowText(WINDOW(EDIT_FILE_PATH), pName);
	CoTaskMemFree(pName);
}

static void OnCommand(HWND hDlg, int id, HWND hwndCtl, UINT codeNotify)
{
	bool handled = false;
	switch(id)
	{
		case BTN_SELECT_FILE:
		{
			GetSaveFileName(hDlg);
			++handled;
		}
		break;
		case BTN_RANDOM_PORT:
		{
			DWORD portNumber = rand() + rand();
			portNumber %= 64535;
			portNumber += 1000;
			SetDlgItemInt(hDlg, EDIT_PORT, portNumber, FALSE);
			++handled;
		}
		break;
		case BTN_CREATE:
		{
			CreateSinkDetails* pSinkDetails = 
				reinterpret_cast<CreateSinkDetails*>(GetWindowLongPtr(hDlg, DWLP_USER));
			HWND hFileText = WINDOW(EDIT_FILE_PATH);
			HWND hPortNumber = WINDOW(EDIT_PORT);
			int fileLength = GetWindowTextLength(hFileText);
			int portLength = GetWindowTextLength(hPortNumber);
			std::wstring file;
			DWORD portNumber = 0;
			if((portLength || fileLength) && !(portLength && fileLength))
			{
				if(portLength)
				{
					portNumber = GetDlgItemInt(hDlg, EDIT_PORT, NULL, FALSE);
				}
				else
				{
					// we have to account for the null while using GetWindowText
					++fileLength;
					file.resize(fileLength);
					file.resize(GetWindowText(hFileText, &file[0], fileLength));
				}
				if((!file.empty()) || portNumber)
				{
					pSinkDetails->file = file;
					pSinkDetails->dwPort = portNumber;
					EndDialog(hDlg, 1);
				}
			}
			++handled;
		}
		break;
		case BTN_CANCELTARGET:
		{
			SendMessage(hDlg, WM_CLOSE, 0, 0);
			++handled;
		}
		break;
	}
	if(!handled)
	{
		switch(codeNotify)
		{
			case EN_UPDATE:
			{
				BOOL enable = (Edit_GetTextLength(WINDOW(EDIT_FILE_PATH)) || Edit_GetTextLength(WINDOW(EDIT_PORT)));
				EnableWindow(WINDOW(BTN_CREATE), enable);
			}
			break;
		}
	}
}

static void OnClose(HWND hDlg)
{
	EndDialog(hDlg, 0);
}

INT_PTR CALLBACK SinkCreateDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch(msg)
	{
		HANDLE_MSG(hDlg, WM_INITDIALOG, &OnInitDlg);
		HANDLE_MSG(hDlg, WM_COMMAND, &OnCommand);
		HANDLE_MSG(hDlg, WM_CLOSE, &OnClose);
	}
	return FALSE;
}
