﻿using System;
using System.Drawing;
using System.Windows.Forms;

namespace AudRecordTestCLI
{
    public partial class AudRecordTestCLIForm : Form
    {
        private AudRecordData recData;
        private AddSink sinkDlg;

        public AudRecordTestCLIForm()
        {
            InitializeComponent();
            ComboCodec.SelectedIndex = ComboMixer.SelectedIndex = ComboConfig.SelectedIndex = -1;
            recData = new AudRecordData();
            sinkDlg = new AddSink();
        }

        private void AudRecordTestCLIForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            sinkDlg.Dispose();
        }

        private void MaybeEnableRecordButton()
        {
            if (
                (ListViewTargets.Items.Count > 0) && 
                (ComboConfig.SelectedIndex != -1) &&
                (ComboMixer.SelectedIndex != -1)
            )
            {
                ButtonRecord.Enabled = true;
            }
        }

        private void AudRecordTestCLIForm_Load(object sender, EventArgs e)
        {
            ComboMixer.DisplayMember = ComboCodec.DisplayMember = "Name";
            ComboMixer.DataSource = recData.Mixers;
            ComboCodec.DataSource = recData.Codecs;
        }

        private void ComboCodec_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboBox comboBox = (ComboBox)sender;
            int index = comboBox.SelectedIndex;
            if (index >= 0)
            {
                ComboConfig.DataSource = recData.GetCodecConfigs(index);
                ComboConfig.DisplayMember = "Name";
                ComboConfig.Enabled = true;
            }
        }

        private void ComboMixer_SelectedIndexChanged(object sender, EventArgs e)
        {
            MaybeEnableRecordButton();
        }

        private void ComboConfig_SelectedIndexChanged(object sender, EventArgs e)
        {
            int codec = ComboCodec.SelectedIndex;
            int config = ComboConfig.SelectedIndex;
            AudRecordCLI.CodecConfig codecConfig = recData.GetCodecConfig(codec, config);
            if (recData.RecordObj == null)
            {
                recData.CreateRecorder(codecConfig);
            }
            else
            {
                recData.RecordObj.ChangeCodec(codecConfig);
            }
            MaybeEnableRecordButton();
        }

        private void ButtonAddSink_Click(object sender, EventArgs e)
        {
            if(sinkDlg.ShowDialog(this) == DialogResult.OK)
            {
                AddSinkResult result = sinkDlg.RealResult;
                IntPtr cookie = IntPtr.Zero;
                string description;
                if (result.file != null)
                {
                    cookie = recData.RecordObj.AddFileTarget(result.file);
                    description = "File - " + result.file;
                }
                else
                {
                    string url = null;
                    cookie = recData.RecordObj.AddNetworkBroadcastTarget(ref result.port, 0, false, ref url);
                    description = "Network port - " + result.port;
                }
                if (cookie != IntPtr.Zero)
                {
                    ListViewItem item = ListViewTargets.Items.Add(description);
                    item.Tag = cookie;
                    MaybeEnableRecordButton();
                }
            }
        }

        private void ListViewTargets_SelectedIndexChanged(object sender, EventArgs e)
        {
            ButtonRemoveSink.Enabled = (((ListView)sender).SelectedItems.Count > 0);
        }

        private void ButtonRemoveSink_Click(object sender, EventArgs e)
        {
            ListViewItem sink = ListViewTargets.SelectedItems[0];
            ListViewTargets.Items.Remove(sink);
            recData.RecordObj.RemoveTarget((IntPtr)sink.Tag);
        }

        private void ButtonRecord_Click(object sender, EventArgs e)
        {
            int mixer = ComboMixer.SelectedIndex;
            recData.Record(mixer);
            ((Button)sender).Enabled = false;
            ButtonPause.Enabled = ButtonStop.Enabled = true;
        }

        private void UpdatePauseButtonText(AudRecordCLI.AudPausedState recording)
        {
            ButtonPause.Text = (recording == AudRecordCLI.AudPausedState.AudPaused) ? "&Pause" : "R&esume";
        }

        private void ButtonPause_Click(object sender, EventArgs e)
        {
            AudRecordCLI.AudPausedState recording = recData.RecordObj.TogglePause();
            UpdatePauseButtonText(recording);
        }

        private void ButtonStop_Click(object sender, EventArgs e)
        {
            recData.RecordObj.Stop();
            ((Button)sender).Enabled = false;
            UpdatePauseButtonText(AudRecordCLI.AudPausedState.AudPaused);
            ButtonPause.Enabled = false;
            ButtonRecord.Enabled = true;
        }
    }
}
