/** \file ETWEvents.h
	\brief Declares functions and macros for use by	the
	Event Tracing for Windows reporting	functionality.
	This file is shared between AudRecordLib and AudRecordCLI
	\cond DEV
*/

#ifndef AUDRECORD_ETWEVENTS_H
#define AUDRECORD_ETWEVENTS_H

#pragma once

#pragma managed(push)
#pragma managed(off)

// both these files contain inline definitions so 
// they're within the pragma block
#include <stddef.h>
#include <strsafe.h>

/** Formats a string and the additional args in a sprintf manner

	\tparam N The inferred size of the buffer
	\param buffer The destination buffer for the printf result
	\param pFormat Printf-esque format string
	\param ... Additional parameters
	\returns The buffer pointer
*/
template<size_t N>
char* LogFormat(char (&buffer)[N], const char* pFormat, ...)
{
	va_list list;
	va_start(list, pFormat);
	StringCchVPrintfA(buffer, N, pFormat, list);
	va_end(list);
	return buffer;
}

void RegisterEventProvider();
void UnregisterEventProvider();
void LogDebugEvent(LPCSTR pszFile, DWORD line, LPCSTR pszFunction, LPCSTR pszData);
void LogSevereEvent(LPCSTR pszFile, DWORD line, LPCSTR pszFunction, LPCSTR pszData);

#pragma managed(pop)

/** \endcond */

#endif
