/** \file alloc.h
	\brief Declarations for functions relating to memory management
	\cond DEV
*/
#ifndef AUDRECORDLIB_ALLOC_H
#define AUDRECORDLIB_ALLOC_H

#include "exports.h"

/** Overrides the default or previous alloc and free functions
	with user defined versions

	\param allocFn The allocation function
	\param freeFn The free function
	\param pData The user data to pass to the functions
	\returns The previous value of the user data, on first call this is the process heap handle
*/
PVOID SetAllocFreeFunctionsInternal(pfnAlloc allocFn, pfnFree freeFn, PVOID pData);

/** Initialises the allocation part of AudRecordLib

	\param hModule The HMODULE of the AudRecord dll, unused
	\returns Whether initialisation was succesful
*/
BOOL InitialiseAlloc(HMODULE hModule);

/** Destroys the allocation info part of AudRecordLib

	\param hModule The HMODULE of the AudRecord dll, unused
	\returns Whether initialisation was succesful
*/
void UninitialiseAlloc(HMODULE hModule);

#ifdef _DEBUG
/** Displays any unfreed allocations allong with their stack trace
	via both the ETW debug stream and OutputDebugString
*/
void DisplayOutstandingAllocs();
#else
#define DisplayOutstandingAllocs() (void)0
#endif

/** \endcond */

#endif
