/** \file exports.cpp
	\brief Implements the exported functions
*/

#include "stdafx.h"
#include "exports.h"
#include "wmfunctions.h"
#include "device.h"
#include "misc.h"
#include "alloc.h"

AUDRECORD_API HRESULT GetWMACodecConfigs(CodecConfigs** ppConfigs)
{
	if(ppConfigs == NULL)
	{
		return E_POINTER;
	}
	RETURN_ON_FAILED_EXP(CheckCOMInitialization());

	CodecConfigs* pConfigs = NULL;
	HRESULT hr = S_OK;
	try
	{
		hr = GetCodecAndProfileDescriptions(pConfigs);
	}
	catch(const std::bad_alloc&)
	{
		DBG_TEXT("Caught bad_alloc");
		hr = E_OUTOFMEMORY;
	}
	catch(...)
	{
		DBG_TEXT("Caught unknown exception");
		hr = E_FAIL;
	}
	if(SUCCEEDED(hr))
	{
		*ppConfigs = pConfigs;
	}
	return hr;
}

AUDRECORD_API STDMETHODIMP_(void) FreeWMACodecConfigs(CodecConfigs* pConfigs)
{
	if(pConfigs)
	{
		FreeCodecAndProfileDescriptions(pConfigs);
	}
}

AUDRECORD_API HRESULT GetAudioEndpoints(EndpointArray** ppEndpoints)
{
	RETURN_ON_FAILED_EXP(CheckCOMInitialization());
	if(ppEndpoints == NULL)
	{
		return E_POINTER;
	}
	EndpointArray* pArray = NULL;
	HRESULT hr = S_OK;
	try
	{
		hr = GetAllAudioEndpoints(pArray);
	}
	catch(const std::bad_alloc&)
	{
		DBG_TEXT("Caught bad_alloc");
		hr = E_OUTOFMEMORY;
	}
	catch(...)
	{
		DBG_TEXT("Caught unknown exception");
		hr = E_FAIL;
	}
	if(SUCCEEDED(hr))
	{
		*ppEndpoints = pArray;
	}
	return hr;
}

AUDRECORD_API void FreeAudioEndpoints(EndpointArray* pEndpoints)
{
	if(pEndpoints)
	{
		FreeAllAudioEndpoints(pEndpoints);
	}
}

AUDRECORD_API HRESULT CreateAudRecordInterface(CodecConfiguration* pCodecConfig, struct IAudRecord** ppAudRecord)
{
	RETURN_ON_FAILED_EXP(CheckCOMInitialization());
	HRESULT hr = E_POINTER;
	if(pCodecConfig && ppAudRecord)
	{
		hr = CreateAudRecordObject(pCodecConfig, ppAudRecord);
	}
	return hr;
}

AUDRECORD_API PVOID SetAllocFreeFunctions(pfnAlloc allocFn, pfnFree freeFn, PVOID pUserData)
{
	PVOID pPrevData = NULL;
	if(allocFn && freeFn)
	{
		pPrevData = SetAllocFreeFunctionsInternal(allocFn, freeFn, pUserData);
	}
	return pPrevData;
}

AUDRECORD_API void DumpUnfreedAllocations()
{
	DisplayOutstandingAllocs();
}